import Circle from '../shapes/Circle';
/**
 * Describes a node of a binary tree
 */
declare class BinaryTreeNode<T extends string | number> {
    /**
     * The value of the node
     */
    value: T;
    /**
     * The canvas circle
     */
    nodeCircle: Circle;
    /**
     * The left child of the node
     */
    left?: BinaryTreeNode<T>;
    /**
     * The right child of the node
     */
    right?: BinaryTreeNode<T>;
    /**
     * For constructing a new binary tree node
     *
     * @param {T} value
     */
    constructor(value: T);
    /**
     * Set the left child
     *
     * @param {BinaryTreeNode} value
     */
    setLeft(value: BinaryTreeNode<T>): void;
    /**
     * Set the right child
     *
     * @param {BinaryTreeNode} value
     */
    setRight(value: BinaryTreeNode<T>): void;
    /**
     * Get the height of the binry tree from the node
     * Height of root is 1
     *
     * @return {number}
     */
    getHeight(): number;
}
export default BinaryTreeNode;
